"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TechnologyStackService = void 0;
var technology_1 = require("../model/technology");
var async_call_blocker_1 = require("../infrastructure/async-call-blocker");
var readJson = require("read-package-json");
var fs = require('fs');
var path = require("path");
// See documentation at https://docs.npmjs.com/cli/v9/configuring-npm/package-lock-json
var TechnologyStackService = /** @class */ (function () {
    function TechnologyStackService() {
        this.technologies = [
            libFor("@angular/core", "Angular Core"),
            libFor("@angular/material", "Angular Material"),
            libFor("@angular/forms", "Angular Forms"),
            libFor("@angular/router", "Angular Router"),
            libFor("rxjs", "RxJS"),
            libFor("@ngrx/effects", "NgRx Effects"),
            libFor("@ngrx/store", "NgRx Store"),
            libFor("@ngrx/router-store", "NgRx Router Store")
        ];
    }
    TechnologyStackService.prototype.scan = function () {
        var technologies = [];
        technologies.push.apply(technologies, this.scanDependencies());
        var technicalNames = new Set();
        technologies.forEach(function (tech) { return technicalNames.add(tech.technicalName); });
        // only add when not already added
        this.scanTransitiveDependencies()
            .filter(function (tech) { return !technicalNames.has(tech.technicalName); })
            .forEach(function (tech) { return technologies.push(tech); });
        technologies.push.apply(technologies, this.scanGitHubActionDependencies()); // no need to check whether already added
        return technologies;
    };
    TechnologyStackService.prototype.scanGitHubActionDependencies = function () {
        try {
            var filename = ".github";
            if (fs.existsSync(filename)) {
                return this.findGitHubDependencies(filename, []);
            }
            else {
                return [];
            }
        }
        catch (err) {
            console.error("Error while scanning github action dependencies: " + err);
            return [];
        }
    };
    TechnologyStackService.prototype.scanTransitiveDependencies = function () {
        try {
            var filename = "package-lock.json";
            if (fs.existsSync(filename)) {
                var packageLockJsonContent = this.readFileContent(filename);
                if (packageLockJsonContent == null) {
                    return [];
                }
                // lockfileVersion = 1 (npm v5 and v6)
                var dependencies = this.getPackageLockDependencies(packageLockJsonContent);
                // lockfileVersion = 2 (npm v7 and v8)
                var packages = this.getPackageLockPackages(packageLockJsonContent);
                var technicalNames_1 = new Set();
                var uniqueTechnologies_1 = [];
                dependencies.forEach(function (dep) {
                    if (dep.technicalName != null
                        && dep.technicalName.length > 0
                        && !technicalNames_1.has(dep.technicalName)) {
                        uniqueTechnologies_1.push(dep);
                        technicalNames_1.add(dep.technicalName);
                    }
                });
                packages.forEach(function (pkg) {
                    if (pkg.technicalName != null
                        && pkg.technicalName.length > 0
                        && !technicalNames_1.has(pkg.technicalName)) {
                        uniqueTechnologies_1.push(pkg);
                        technicalNames_1.add(pkg.technicalName);
                    }
                });
                return uniqueTechnologies_1;
            }
            else {
                console.log('No package-lock.json, skip scanning of transitive dependencies');
                return [];
            }
        }
        catch (err) {
            console.error("Error while scanning transitive dependencies: " + err);
            return [];
        }
    };
    TechnologyStackService.prototype.getPackageLockDependencies = function (packageLockJsonContent) {
        var _this = this;
        if (packageLockJsonContent.dependencies == null) {
            return [];
        }
        var techs = [];
        Object.keys(packageLockJsonContent.dependencies).forEach(function (key) {
            var dependencyObject = packageLockJsonContent.dependencies[key];
            var isDevDependency = dependencyObject["dev"];
            var isOptionalDependency = dependencyObject["optional"];
            if ((isDevDependency != null && isDevDependency)
                || (isOptionalDependency != null && isOptionalDependency)) {
                // skip
            }
            else {
                var tech = __assign(__assign({}, (0, technology_1.createTechnology)()), { version: dependencyObject["version"], technicalName: key, transitive: true });
                var lib = _this.technologies.find(function (tech) { return tech.technicalName === key; });
                if (!!lib) {
                    tech.wellKnown = true;
                    tech.displayName = lib.displayName;
                }
                techs.push(tech);
            }
        });
        return techs;
    };
    TechnologyStackService.prototype.getPackageLockPackages = function (packageLockJsonContent) {
        var _this = this;
        if (packageLockJsonContent.packages == null) {
            return [];
        }
        var techs = [];
        Object.keys(packageLockJsonContent.packages).forEach(function (key) {
            var dependencyObject = packageLockJsonContent.packages[key];
            // packages typically start with node_modules/ (unlike the lockFileVersion=1 dependencies)
            var cleanKey = key;
            if (key != null && key.lastIndexOf("node_modules/") >= 0) {
                // sometimes node_modules can be found twice
                var startIndex = key.lastIndexOf("node_modules/");
                cleanKey = key.substring(startIndex + 13);
            }
            var isDevDependency = dependencyObject["dev"];
            var isOptionalDependency = dependencyObject["optional"];
            var isDevOptionalDependency = dependencyObject["devOptional"];
            if ((isDevDependency != null && isDevDependency)
                || (isOptionalDependency != null && isOptionalDependency)
                || (isDevOptionalDependency != null && isDevOptionalDependency)) {
                // skip
            }
            else {
                var tech = __assign(__assign({}, (0, technology_1.createTechnology)()), { version: dependencyObject["version"], technicalName: cleanKey, transitive: true });
                var lib = _this.technologies.find(function (tech) { return tech.technicalName === key; });
                if (!!lib) {
                    tech.wellKnown = true;
                    tech.displayName = lib.displayName;
                }
                techs.push(tech);
            }
        });
        return techs;
    };
    TechnologyStackService.prototype.scanDependencies = function () {
        var _this = this;
        var packageJsonContent = this.readPackageJson("package.json");
        if (packageJsonContent == null || packageJsonContent.dependencies == null) {
            return [];
        }
        return Object.keys(packageJsonContent.dependencies).map(function (key) {
            var version = packageJsonContent.dependencies[key];
            var tech = __assign(__assign({}, (0, technology_1.createTechnology)()), { version: version, technicalName: key });
            var lib = _this.technologies.find(function (tech) { return tech.technicalName === key; });
            if (!!lib) {
                tech.wellKnown = true;
                tech.displayName = lib.displayName;
            }
            return tech;
        });
    };
    TechnologyStackService.prototype.findGitHubDependencies = function (dirPath, arrayOfTechs) {
        var _this = this;
        var files = fs.readdirSync(dirPath);
        arrayOfTechs = arrayOfTechs || [];
        files.forEach(function (file) {
            if (fs.statSync(dirPath + "/" + file).isDirectory()) {
                arrayOfTechs = _this.findGitHubDependencies(dirPath + "/" + file, arrayOfTechs);
            }
            else {
                if (file.toUpperCase().endsWith('.YML')
                    || file.toUpperCase().endsWith('.YAML')) {
                    var techsInAction = _this.extractTechnologiesFromGitHubAction(dirPath + "/" + file);
                    techsInAction.forEach(function (techInAction) {
                        var existingTech = arrayOfTechs.find(function (t) { return t != null && t.technicalName === techInAction.technicalName && t.version === techInAction.version; });
                        if (existingTech == null) {
                            arrayOfTechs.push(techInAction);
                        }
                    });
                }
            }
        });
        return arrayOfTechs;
    };
    TechnologyStackService.prototype.extractTechnologiesFromGitHubAction = function (file) {
        var yamlContent = fs.readFileSync(file, 'utf8').toString();
        var techs = [];
        yamlContent.split('\n')
            .map(function (line) { return line.replace('\r', '').replace('\t', ''); })
            .map(function (line) { return line.trim(); })
            .filter(function (line) { return !line.startsWith('#'); })
            .filter(function (line) { return line.startsWith('uses:') || line.startsWith('- uses:'); })
            .forEach(function (line) {
            var index = line.indexOf('uses:');
            var value = line.substring(index + 5).trim();
            var parts = value.split('@');
            techs.push(__assign(__assign({}, (0, technology_1.createTechnology)()), { technicalName: parts[0], version: (parts.length > 1) ? parts[1] : null }));
        });
        return techs;
    };
    TechnologyStackService.prototype.readFileContent = function (file) {
        // Directly using fs instead of readJson, because readJson would do unwanted extra Json parsing
        return JSON.parse(fs.readFileSync(file, 'utf8').toString());
    };
    TechnologyStackService.prototype.readPackageJson = function (file) {
        var fileContent = null;
        readJson(file, null, false, function (er, data) {
            if (er) {
                console.error("Error: There was an error reading " + file);
                return;
            }
            fileContent = data;
        });
        // Block until file is read
        while (!fileContent) {
            (0, async_call_blocker_1.blockExecution)();
        }
        return fileContent;
    };
    return TechnologyStackService;
}());
exports.TechnologyStackService = TechnologyStackService;
function libFor(technicalName, displayName) {
    return { displayName: displayName, technicalName: technicalName };
}
